/****************************************************************************

    This file is part of The Newtonator soft synth.

    Copyright (C) 2010-2011 Michael Bechard <mbechard@users.sourceforge.net>

    The Newtonator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The Newtonator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Newtonator.  If not, see <http://www.gnu.org/licenses/>.

****************************************************************************/

#include "newt_scalar_widget.h"
#include "newt_parent_widget.h"
#include <glib-object.h>
#include <iostream>

using namespace std;

NewtScalarWidget::NewtScalarWidget(Glib::ustring label, double min, double max, double def, NewtParentWidget* parent, ControlEnum ctrlID, unsigned int precision, double climbRate)
: Gtk::HBox(false, 5)
, m_value(def)
, m_min(min)
, m_max(max)
, m_precision(precision)
, m_climbRate(climbRate)
, m_spin(climbRate, precision)
, m_label(label)
, m_apple(*this)
, m_ctrlID(ctrlID)
, m_parent(parent)
{
    //initialize this container

    //on_size_request
    set_border_width(3);

    //initialize children
    InitSpin();
    InitApple();

    //add children to container
    pack_start(m_label, false, false);
    pack_start(m_spin, false, false);
    pack_end(m_apple, true, true);

    SetValue(def);
    if (m_parent)
        m_parent->signal_value_should_change().connect(sigc::mem_fun(*this, &NewtScalarWidget::on_value_should_change));
}

NewtScalarWidget::~NewtScalarWidget()
{
    //dtor
}

void NewtScalarWidget::InitSpin()
{
    m_spin.set_range(m_min, m_max);
    m_spin.set_update_policy(Gtk::UPDATE_IF_VALID);
    m_spin.set_increments(m_climbRate, m_climbRate * 10);
    m_spin.signal_value_changed().connect(sigc::mem_fun(*this, &NewtScalarWidget::on_spin_value_changed));
}

void NewtScalarWidget::InitApple()
{
    m_spin.signal_value_changed().connect(sigc::mem_fun(m_apple, &AppleWidget::on_spin_value_changed));
}

void NewtScalarWidget::on_spin_value_changed()
{
    m_value = m_spin.get_value();
    m_valueChanged.emit();
    if (m_parent)
        m_parent->EmitValueChange(m_ctrlID, m_value);
}

void NewtScalarWidget::on_value_should_change(ControlEnum ctrl, stk::StkFloat val)
{
    if (ctrl == m_ctrlID)
        SetValue(val);
}

void NewtScalarWidget::forall_vfunc(gboolean include_internals, GtkCallback callback, gpointer callback_data)
{
    callback(reinterpret_cast<GtkWidget*>(m_label.gobj()), callback_data);
    callback(reinterpret_cast<GtkWidget*>(m_spin.gobj()), callback_data);
    callback(reinterpret_cast<GtkWidget*>(m_apple.gobj()), callback_data);
}

void NewtScalarWidget::SetValue(double val)
{
    m_spin.set_value(val);
}
