/****************************************************************************
    
    This file is part of The Newtonator soft synth.

    Copyright (C) 2010-2011 Michael Bechard <mbechard@users.sourceforge.net>

    The Newtonator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The Newtonator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Newtonator.  If not, see <http://www.gnu.org/licenses/>.

****************************************************************************/

#include "sensitivity_container.h"
#include "config.h"

SensitivityContainer::SensitivityContainer(bool initialSensitivity)
{
    m_sensitive = initialSensitivity;
    m_parent = NULL;
}

SensitivityContainer::~SensitivityContainer()
{
    //dtor
}

void SensitivityContainer::SetSensitivity(bool sensitive)
{
    if (m_sensitive == sensitive)
        return;

    m_sensitive = sensitive;

    CheckSensitivity();
}

void SensitivityContainer::CheckSensitivity()
{
    TypeWidgets::iterator wit;
    TypeContainers::iterator cit;

    if (m_sensitive && ((m_parent && m_parent->GetSensitivity()) || m_parent == NULL))
    {
        for (wit = m_widgets.begin(); wit != m_widgets.end(); wit++)
        {
            //TRACE2("Setting sensitivity true", this, (*wit)->get_name());
            (*wit)->set_sensitive(true);
        }
    }
    else
    {
        for (wit = m_widgets.begin(); wit != m_widgets.end(); wit++)
        {
            //TRACE2("Setting sensitivity false", this, (*wit)->get_name());
            (*wit)->set_sensitive(false);
        }
    }

    for (cit = m_containers.begin(); cit != m_containers.end(); cit++)
    {
        (*cit)->CheckSensitivity();
    }
}

bool SensitivityContainer::GetSensitivity()
{
    if (!m_sensitive || (m_parent && !m_parent->GetSensitivity()))
    {
        return false;
    }

    return true;
}
